#include "hlhdf_debug.h"
#include "hdf5.h"
#include <time.h>
#include <stdio.h>

hlhdf_debug_struct hlhdfDbg; /* = { HLHDF_INFO, HL_DefaultDebugFunction};*/

static void setLogTime(char* strtime, int len)
{
   time_t cur_time;
   struct tm* tu_time;

   time(&cur_time);
   tu_time = gmtime(&cur_time);
   strftime(strtime,len,"%Y/%m/%d %H:%M:%S", tu_time);   
}

void HL_InitializeDebugger()
{
   hlhdfDbg.dbgLevel = HLHDF_SILENT;
   hlhdfDbg.dbgfun = HL_DefaultDebugFunction;
   hlhdfDbg.hdf5showerror = 1;
   hlhdfDbg.hdf5fun = HL_DefaultHdf5ErrorFunction;
}

void HL_DefaultDebugFunction(char* filename, int lineno, HL_Debug lvl, const char* fmt, ...)
{
   char msgbuff[512];
   char dbgtype[20];
   char strtime[24];
   char infobuff[120];
   
   va_list alist;
   va_start(alist,fmt);

   if(hlhdfDbg.dbgLevel==HLHDF_SILENT)
      return;

   setLogTime(strtime,24);

   sprintf(dbgtype,"");
   
   if(lvl>=hlhdfDbg.dbgLevel) {
      switch(lvl) {
      case HLHDF_DEBUG:
	 sprintf(dbgtype,"DEBUG");
	 break;
      case HLHDF_DEPRECATED:
	 sprintf(dbgtype,"DEPRECATED");
	 break;
      case HLHDF_INFO:
	 sprintf(dbgtype,"INFO");
	 break;
      case HLHDF_WARNING:
	 sprintf(dbgtype,"WARNING");
	 break;
      case HLHDF_ERROR:
	 sprintf(dbgtype,"ERROR");
	 break;
      case HLHDF_CRITICAL:
	 sprintf(dbgtype,"CRITICAL");
	 break;
      default:
	 sprintf(dbgtype,"UNKNOWN");
	 break;
      }
   } else {
      return;
   }
   sprintf(infobuff,"%20s : %11s",strtime,dbgtype);
   vsprintf(msgbuff,fmt,alist);

   fprintf(stderr,"%s : %s (%s:%d)\n",infobuff,msgbuff,filename,lineno);
}

void HL_DefaultHdf5ErrorFunction(int n, H5E_error_t* rowmsg)
{
   if(hlhdfDbg.hdf5showerror) {
      fprintf(stderr,"  HDF5-ERROR: #%03d: %s line %d in %s: %s\n",
	      n,rowmsg->file_name,rowmsg->line,rowmsg->func_name,rowmsg->desc);
      fprintf(stderr,"    major(%d): %s\n",rowmsg->maj_num,H5Eget_major(rowmsg->maj_num));
      fprintf(stderr,"    minor(%d): %s\n",rowmsg->min_num,H5Eget_minor(rowmsg->min_num));
   }
}

void HL_setDebugLevel(HL_Debug lvl)
{
   hlhdfDbg.dbgLevel = lvl;
}

void HL_setDebugFunction(void (*dbgfun)(char* filename, int lineno, HL_Debug lvl, const char* fmt, ...))
{
   hlhdfDbg.dbgfun = dbgfun;
}

void HL_disableHdf5ErrorReporting()
{
   hlhdfDbg.hdf5showerror = 0;
}

void HL_enableHdf5ErrorReporting()
{
   hlhdfDbg.hdf5showerror = 1;
}


void HL_setHdf5ErrorFunction(void (*hdf5fun)(int n, H5E_error_t*))
{
   hlhdfDbg.hdf5fun = hdf5fun;
}

static herr_t HDF5ErrorWalker(int n, H5E_error_t *err_desc, void *client_data)
{
   hlhdfDbg.hdf5fun(n,err_desc);
   return 0;
}
			     
hid_t HL_hdf5_debug_function(void* inv)
{
   H5Ewalk(H5E_WALK_DOWNWARD,HDF5ErrorWalker,NULL);
   return 0;
}
