#ifndef HLHDF_DEBUG_H
#define HLHDF_DEBUG_H

#include <stdarg.h>
#include <H5Epublic.h>

typedef enum HL_Debug { HLHDF_SPEWDEBUG=0, HLHDF_DEBUG, HLHDF_DEPRECATED,HLHDF_INFO,HLHDF_WARNING,HLHDF_ERROR,HLHDF_CRITICAL,HLHDF_SILENT } HL_Debug;

typedef struct {
   HL_Debug dbgLevel;
   void (*dbgfun)(char* filename, int lineno, HL_Debug lvl, const char* fmt,...);
   int hdf5showerror;
   void (*hdf5fun)(int n, H5E_error_t* rowmsg);
} hlhdf_debug_struct;

extern hlhdf_debug_struct hlhdfDbg;

void HL_DefaultDebugFunction(char* filename, int lineno, HL_Debug lvl, const char* fmt, ...);
void HL_DefaultHdf5ErrorFunction(int n, H5E_error_t* rowmsg);

void HL_InitializeDebugger(void);
void HL_setDebugLevel(HL_Debug lvl);
void HL_setDebugFunction(void (*dbgfun)(char* filename, int lineno, HL_Debug lvl, const char* fmt, ...));
void HL_setHdf5ErrorFunction(void (*hdf5fun)(int n, H5E_error_t* val));
void HL_disableHdf5ErrorReporting(void);
void HL_enableHdf5ErrorReporting(void);

/* This is the debug function that is used to forward messages
 * to the appropriate debugger function
 */
hid_t HL_hdf5_debug_function(void* inv);

/* A Chunk with various macros for the different HL_Debug levels, I really wish that other c-compilers than gcc
* would take variable argument lists in the #define macro!!!*/

#ifdef DEBUG_HLHDF
#define HL_SPEWDEBUG0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_SPEWDEBUG,msg)

#define HL_SPEWDEBUG1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_SPEWDEBUG,msg,arg1)

#define HL_SPEWDEBUG2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_SPEWDEBUG,msg,arg1,arg2)

#define HL_SPEWDEBUG3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_SPEWDEBUG,msg,arg1,arg2,arg3)

#define HL_SPEWDEBUG4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_SPEWDEBUG,msg,arg1,arg2,arg3,arg4)

#define HL_DEBUG0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEBUG,msg)

#define HL_DEBUG1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEBUG,msg,arg1)

#define HL_DEBUG2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEBUG,msg,arg1,arg2)

#define HL_DEBUG3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEBUG,msg,arg1,arg2,arg3)

#define HL_DEBUG4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEBUG,msg,arg1,arg2,arg3,arg4)

#define HL_DEPRECATED0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEPRECATED,msg)

#define HL_DEPRECATED1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEPRECATED,msg,arg1)

#define HL_DEPRECATED2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEPRECATED,msg,arg1,arg2)

#define HL_DEPRECATED3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEPRECATED,msg,arg1,arg2,arg3)

#define HL_DEPRECATED4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_DEPRECATED,msg,arg1,arg2,arg3,arg4)

#define HL_INFO0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg)

#define HL_INFO1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1)

#define HL_INFO2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1,arg2)

#define HL_INFO3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1,arg2,arg3)

#define HL_INFO4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1,arg2,arg3,arg4)

#define HL_WARNING0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg)

#define HL_WARNING1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1)

#define HL_WARNING2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1,arg2)

#define HL_WARNING3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1,arg2,arg3)

#define HL_WARNING4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1,arg2,arg3,arg4)

#define HL_ERROR0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg)

#define HL_ERROR1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1)

#define HL_ERROR2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1,arg2)

#define HL_ERROR3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1,arg2,arg3)

#define HL_ERROR4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1,arg2,arg3,arg4)

#define HL_CRITICAL0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg)

#define HL_CRITICAL1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1)

#define HL_CRITICAL2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1,arg2)

#define HL_CRITICAL3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1,arg2,arg3)

#define HL_CRITICAL4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1,arg2,arg3,arg4)

/* No macro like HL_SILENT since that would ruin the possibility to have a silent option :) */
#else
/* In production environment we dont want a lot of unessecary calls so these are left blank... */
#define HL_SPEWDEBUG0(msg)

#define HL_SPEWDEBUG1(msg,arg1)

#define HL_SPEWDEBUG2(msg,arg1,arg2)

#define HL_SPEWDEBUG3(msg,arg1,arg2,arg3)

#define HL_SPEWDEBUG4(msg,arg1,arg2,arg3,arg4)

#define HL_DEBUG0(msg)

#define HL_DEBUG1(msg,arg1)

#define HL_DEBUG2(msg,arg1,arg2)

#define HL_DEBUG3(msg,arg1,arg2,arg3)

#define HL_DEBUG4(msg,arg1,arg2,arg3,arg4)

#define HL_DEPRECATED0(msg)

#define HL_DEPRECATED1(msg,arg1)

#define HL_DEPRECATED2(msg,arg1,arg2)

#define HL_DEPRECATED3(msg,arg1,arg2,arg3)

#define HL_DEPRECATED4(msg,arg1,arg2,arg3,arg4)

#define HL_INFO0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg)

#define HL_INFO1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1)

#define HL_INFO2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1,arg2)

#define HL_INFO3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1,arg2,arg3)

#define HL_INFO4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_INFO,msg,arg1,arg2,arg3,arg4)

#define HL_WARNING0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg)

#define HL_WARNING1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1)

#define HL_WARNING2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1,arg2)

#define HL_WARNING3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1,arg2,arg3)

#define HL_WARNING4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_WARNING,msg,arg1,arg2,arg3,arg4)

#define HL_ERROR0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg)

#define HL_ERROR1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1)

#define HL_ERROR2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1,arg2)

#define HL_ERROR3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1,arg2,arg3)

#define HL_ERROR4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_ERROR,msg,arg1,arg2,arg3,arg4)

#define HL_CRITICAL0(msg) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg)

#define HL_CRITICAL1(msg,arg1) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1)

#define HL_CRITICAL2(msg,arg1,arg2) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1,arg2)

#define HL_CRITICAL3(msg,arg1,arg2,arg3) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1,arg2,arg3)

#define HL_CRITICAL4(msg,arg1,arg2,arg3,arg4) \
hlhdfDbg.dbgfun(__FILE__,__LINE__,HLHDF_CRITICAL,msg,arg1,arg2,arg3,arg4)


#endif

#endif
