/* --------------------------------------------------------------------------
$Id: write_vhlhdf.c,v 1.3 2004/09/20 09:58:16 ahenja Exp $
Program:        

Description:    Functions for writing an HDF5 file with the HighLevel
		protocol.

Author(s):      Anders Henja

Copyright:	Swedish Meteorological and Hydrological Institute, 2000

History:
$Log: write_vhlhdf.c,v $
Revision 1.3  2004/09/20 09:58:16  ahenja
Fixed strange bug!?!?!?!
:wq!

Revision 1.2  2004/03/18 11:53:50  ahenja
Modified with support for szip, also changed so that it is possible
to specify HL_FileCreationProperty when creating a file. Also changed
how the compression handling is working. Added a nicer way to
perform debugging and error handling. Now it is possible to define
an own error handler and hook it into the system, if this is done
all error messages will be sent to the given function.

Revision 1.1.1.1  2001/12/14 12:02:41  ahenja
First CVS version of hlhdf

Revision 1.10  2001/12/07 13:20:32  ahenja
Updated with possibilities to read and write references.

Revision 1.9  2001/11/09 08:30:56  ahenja
Updated with possibility to write references, unfortunately i havent got the time to implement read support.

Revision 1.8  2001/03/13 19:59:23  adybbroe
*** empty log message ***

Revision 1.7  2001/01/18 12:39:54  ahenja
*** empty log message ***

Revision 1.6  2000/03/22 08:43:06  ahenja
Updated with spell checking and header information.

-----------------------------------------------------------------------------*/
#include "write_vhlhdf.h"
#include "write_hlhdf.h"
#include "hlhdf_debug.h"

int doWriteHdf5Attribute(hid_t rootGrp, HL_Node* parentNode, char* parentName,
			 HL_Node* childNode, char* childName)
{
   hid_t tmpLocId=-1;
   HL_SPEWDEBUG0("ENTER: doWriteHdf5Attribute");
   if(!parentName || !childName) {
      HL_ERROR0("Can't write HDF5 group since either parentName or childName is NULL");
      return 0;
   }

   if(strcmp(parentName,"")==0) {
      tmpLocId=rootGrp;
   } else {
      tmpLocId=parentNode->hdfId;
   }

   if(strcmp(childNode->format,DATAFORMAT_UNDEFINED)!=0 && childNode->typeId<0) {
      childNode->typeId = translateCharToDatatype(childNode->format);
   }
   if(childNode->typeId<0) {
      HL_ERROR2("Can't recognize datatype '%s' or typeId='%ld'", \
		 childNode->format,(long)childNode->typeId);
      return 0;
   }

   if(childNode->ndims==0) {
      if(writeScalarDataAttribute(tmpLocId, childNode->typeId, childName, childNode->data)<0) {
	 HL_ERROR1("Failed to write scalar data attribute '%s'",childNode->name);
	 return 0;
      }
   } else {
      if(writeSimpleDataAttribute(tmpLocId,childNode->typeId, childName,
				  childNode->ndims, childNode->dims,
				  childNode->data)<0) {
	 HL_ERROR1("Failed to write simple data attribute '%s'",childNode->name);
	 return 0;
      }
   }

   return 1;
}

int doWriteHdf5Group(hid_t rootGrp, HL_Node* parentNode, char* parentName,
		     HL_Node* childNode, char* childName)
{
   HL_SPEWDEBUG0("ENTER: doWriteHdf5group");
   if(!parentName || !childName) {
      HL_ERROR0("Can't write HDF5 group since either parentName or childName is NULL");
      return 0;
   }
   if(strcmp(parentName,"")==0) {
      if(childNode->hdfId>=0) {
	 H5Gclose(childNode->hdfId);
	 childNode->hdfId=-1;
      }
      childNode->hdfId=H5Gcreate(rootGrp,childName,0);
   } else {
      if(childNode->hdfId>=0) {
	 H5Gclose(childNode->hdfId);
	 childNode->hdfId=-1;
      }
      childNode->hdfId=H5Gcreate(parentNode->hdfId,childName,0);
   }
   if(childNode->hdfId<0) {
      HL_ERROR1("Failed to create group %s",childNode->name);
      return 0;
   }
   return 1;
}

int doWriteHdf5HLDataset(hid_t rootGrp, HL_Node* parentNode, char* parentName,
			 HL_Node* childNode, char* childName, HL_Compression* compression)
{
   hid_t tmpLocId=-1;
   HL_SPEWDEBUG0("ENTER: doWriteHdf5HLDataset");
   if(!parentName || !childName) {
      HL_ERROR0("Can't write HDF5 dataset since either parentName or childName is NULL");
      return 0;
   }
   if(strcmp(parentName,"")==0) {
      tmpLocId=rootGrp;
   } else {
      tmpLocId=parentNode->hdfId;
   }

   if(strcmp(childNode->format,DATAFORMAT_UNDEFINED)!=0 && childNode->typeId<0) {
      childNode->typeId = translateCharToDatatype(childNode->format);
   }
   if(childNode->typeId<0) {
      HL_ERROR2("Can't recognize datatype '%s' or typeId='%ld'", \
		 childNode->format,(long)childNode->typeId);
      return 0;
   }
   if(childNode->hdfId>=0) {
      H5Dclose(childNode->hdfId);
      childNode->hdfId=-1;
   }   
   childNode->hdfId = createSimpleHLDataset(tmpLocId, childNode->typeId,
					    childName, childNode->ndims,
					    childNode->dims,
					    childNode->data, compression);
   if(childNode->hdfId<0) {
      HL_ERROR1("Failed to create dataset %s",childNode->name);
      return 0;
   }

   return 1;   
}

int doWriteHdf5Dataset(hid_t rootGrp, HL_Node* parentNode, char* parentName,
		       HL_Node* childNode, char* childName, int doCompress)
{
   hid_t tmpLocId=-1;
   HL_SPEWDEBUG0("ENTER: doWriteHdf5Dataset");
   if(!parentName || !childName) {
      HL_ERROR0("Can't write HDF5 dataset since either parentName or childName is NULL");
      return 0;
   }
   if(strcmp(parentName,"")==0) {
      tmpLocId=rootGrp;
   } else {
      tmpLocId=parentNode->hdfId;
   }

   if(strcmp(childNode->format,DATAFORMAT_UNDEFINED)!=0 && childNode->typeId<0) {
      childNode->typeId = translateCharToDatatype(childNode->format);
   }
   if(childNode->typeId<0) {
      HL_ERROR2("Can't recognize datatype '%s' or typeId='%ld'", \
		 childNode->format,(long)childNode->typeId);
      return 0;
   }
   if(childNode->hdfId>=0) {
      H5Dclose(childNode->hdfId);
      childNode->hdfId=-1;
   }
   childNode->hdfId = createSimpleDataset(tmpLocId, childNode->typeId,
					  childName, childNode->ndims,
					  childNode->dims,
					  childNode->data, doCompress);
   if(childNode->hdfId<0) {
      HL_ERROR1("Failed to create dataset %s",childNode->name);
      return 0;
   }

   return 1;   
}

int doCommitHdf5Datatype(hid_t loc_id, HL_Node* parentNode, char* parentName,
			 HL_Node* childNode, char* childName)
{
   HL_DEBUG0("ENTER: doCommitHdf5Datatype");
   if(loc_id<0) {
      HL_ERROR0("Trying to create a committed datatype without setting typeid");
      return 0;
   }

   if((commitType(loc_id,childNode->name,childNode->hdfId))<0)
      return 0;
   if(H5Tcommitted(childNode->hdfId)<=0) {
      HL_ERROR1("Failed to commit datatype'%s'",childNode->name);
   }

   return 1;
}

int doWriteHdf5Reference(hid_t rootGrp, hid_t file_id, HL_Node* parentNode,
			 char* parentName, HL_Node* childNode, char* childName)
{
   hid_t tmpLocId=-1;
   HL_DEBUG0("ENTER: doWriteHdf5Reference");
   if(!parentName || !childName) {
      HL_ERROR0("Can't write HDF5 dataset since either parentName or childName is NULL");
      return 0;
   }
   if(strcmp(parentName,"")==0) {
      tmpLocId=rootGrp;
   } else {
      tmpLocId=parentNode->hdfId;
   }
    if(createReference(tmpLocId,file_id, childName, (char*)childNode->data)<0) {
      HL_ERROR3("Failed to create reference from '%s/%s' to '%s'", \
		parentName,childName, (char*)childNode->data);
      return 0;
   }
   return 1;
}

int writeHL_NodeList(HL_NodeList* nodelist, HL_FileCreationProperty* property, HL_Compression* compression)
{
   int i;
   HL_Node* parentNode=NULL;
   char parentName[256];
   char childName[256];
   hid_t file_id=-1;
   hid_t gid=-1;
   hid_t strhid=-1;
   HL_DEBUG0("ENTER: writeHL_NodeList");
   if((file_id=createHlHdfFileWithProperty(nodelist->filename,property))<0) {
      return 0;
   }
   
   if((gid=H5Gopen(file_id,"."))<0) {
      H5Fclose(file_id);
      return 0;
   }

   for(i=0;i<nodelist->nNodes;i++) {
      if(!extractParentChildName(nodelist->nodes[i],parentName,childName)) {
	 HL_ERROR0("Failed to extract parent, child name");
	 goto fail;
      }
      if((strcmp(parentName,"")!=0) && !(parentNode = getNode(nodelist,parentName))) {
	 HL_ERROR1("Failed to locate parent node '%s'",parentName);
	 goto fail;
      }
      switch(nodelist->nodes[i]->type) {
      case ATTRIBUTE_ID: {
	 if(!doWriteHdf5Attribute(gid,parentNode,parentName,nodelist->nodes[i],childName)) {
	    goto fail;
	 }
	 break;
      }
      case GROUP_ID: {
	 if(!doWriteHdf5Group(gid,parentNode, parentName, nodelist->nodes[i], childName)) { 
	    goto fail;
	 }
	 break;
      }
      case DATASET_ID: {
	 if(compression!=NULL) {
	    if(!doWriteHdf5HLDataset(gid,parentNode, parentName, nodelist->nodes[i],childName,compression)) {
	       goto fail;
	    }
	 } else {
	    if(!doWriteHdf5HLDataset(gid,parentNode, parentName, nodelist->nodes[i],childName,nodelist->nodes[i]->compression)) {
	       goto fail;
	    }
	 }
	 break;
      }
      case TYPE_ID: {
	 if(!doCommitHdf5Datatype(file_id,parentNode, parentName, nodelist->nodes[i],childName))
	    goto fail;
	 break;
      }
      case REFERENCE_ID: {
	 if(!doWriteHdf5Reference(gid,file_id,parentNode, parentName, nodelist->nodes[i],childName))
	    goto fail;
	 break;
      }
      default: {
	 HL_ERROR0("Unrecognized type");
	 break;
      }
      }
   }

   if(gid>=0) {
      H5Gclose(gid);
   }
   if(file_id>=0) {
      H5Fclose(file_id);
   }
   if(strhid>=0) {
      H5Tclose(strhid);
   }
   return 1;
fail:
   if(gid>=0)
      H5Gclose(gid);
   if(file_id>=0)
      H5Fclose(file_id);
   if(strhid>=0)
      H5Tclose(strhid);
   return 0;   
}

int writeNodeList(HL_NodeList* nodelist, int doCompress)
{
   HL_Compression compr;
   HL_DEBUG0("ENTER: writeNodeList");
   initHL_Compression(&compr,CT_ZLIB);
   compr.level=doCompress;
   return writeHL_NodeList(nodelist,NULL,&compr);
}

