############################################################
#
# $Id: regression_test.py,v 1.1.1.1 2001/12/14 12:02:45 ahenja Exp $
#
# Description:	Regression test script for pyhl and related HL-HDF
#		functionality.
#
# Author(s):	Anders Henja
#
# Copyright:	Swedish Meteorological and Hydrological Institute, 2001
#
# History:	2001-11-07 Created by Anders Henja
############################################################
import _pyhl
from Numeric import *
import os

def write_int_list():
    a=_pyhl.nodelist()
    b=_pyhl.node(_pyhl.GROUP_ID,"/info")
    a.addNode(b)
    b=_pyhl.node(_pyhl.ATTRIBUTE_ID,"/info/list")
    c=[10,20,30,40]
    b.setArrayValue(-1,[4],c,"int",-1)
    a.addNode(b)
    a.write("ahetesthdf54_int_list.hdf")

def verify_int_list():
    a=_pyhl.read_nodelist("ahetesthdf54_int_list.hdf")
    a.selectAll()
    a.fetch()
    b=a.getNode("/info/list")
    # .data() will return an array of correct type instead
    # of the inserted list since it is impossible to know
    # if it was stored as an array or a list.
    if `b.data()` == `array([10,20,30,40]).astype('i')`:
	return 1
    else:
	return 0

def write_string_list():
    a=_pyhl.nodelist()
    b=_pyhl.node(_pyhl.ATTRIBUTE_ID,"/strlist")
    c=["Strang 1","Strang 2", "Strang 3", "Strang 4"]
    b.setArrayValue(-1,[4],c,"string",-1)
    a.addNode(b)
    a.write("ahetesthdf54_string_list.hdf")

def verify_string_list():
    a=_pyhl.read_nodelist("ahetesthdf54_string_list.hdf")
    a.selectAll()
    a.fetch()
    b=a.getNode("/strlist")
    if `b.data()` == `["Strang 1","Strang 2", "Strang 3", "Strang 4"]`:
	return 1
    else:
	return 0

def get_uchar_array():
    c=arange(110)
    c=array(c.astype('b'),'b')
    c=reshape(c,(10,11)).astype('b')
    return c

def get_short_array():
    c=arange(120)
    c=array(c.astype('s'),'s')
    c=reshape(c,(10,12)).astype('s')
    return c

def get_int_array():
    c=arange(100)
    c=array(c.astype('i'),'i')
    c=reshape(c,(10,10)).astype('i')
    return c

def get_float_array():
    c=arange(100)
    c=array(c.astype('f'),'f')
    c=c*0.1
    c=reshape(c,(10,10)).astype('f')
    return c    

def write_array_list():
    a=_pyhl.nodelist()
    b=_pyhl.node(_pyhl.DATASET_ID,"/uchardata")
    c=get_uchar_array()
    b.setArrayValue(1,[10,11],c,"uchar",-1)
    a.addNode(b)
    b=_pyhl.node(_pyhl.DATASET_ID,"/shortdata")
    c=get_short_array()
    b.setArrayValue(1,[10,12],c,"short",-1)
    a.addNode(b)
    b=_pyhl.node(_pyhl.DATASET_ID,"/intdata")
    c=get_int_array()
    b.setArrayValue(1,[10,10],c,"int",-1)
    a.addNode(b)
    b=_pyhl.node(_pyhl.DATASET_ID,"/floatdata")
    c=get_float_array()
    b.setArrayValue(1,[10,10],c,"float",-1)
    a.addNode(b)    
    a.write("ahetesthdf54_arrays.hdf")

def verify_array_list():
    a=_pyhl.read_nodelist("ahetesthdf54_arrays.hdf")
    seems_ok=1
    a.selectAll()
    a.fetch()
    c=a.getNode("/uchardata")
    if `c.data()`!=`get_uchar_array()`:
	seems_ok=0
    c=a.getNode("/shortdata")
    if `c.data()`!=`get_short_array()`:
	seems_ok=0
    c=a.getNode("/intdata")
    if `c.data()`!=`get_int_array()`:
	seems_ok=0    
    c=a.getNode("/floatdata")
    if `c.data()`!=`get_float_array()`:
	seems_ok=0    
    return seems_ok

def write_compound_object():
    import _rave_info_type
    rinfo_type = _rave_info_type.type()
    rinfo_obj=_rave_info_type.object()
    rinfo_obj.xsize=10
    rinfo_obj.ysize=11
    rinfo_obj.xscale=140.0
    rinfo_obj.yscale=160.0
    area_extent = [10.0,20.0,30.0,40.0]
    rinfo_obj.area_extent=tuple(area_extent)
    a=_pyhl.nodelist()

    b=_pyhl.node(_pyhl.TYPE_ID,"/MinDatatyp")
    b.commit(rinfo_type.hid())
    a.addNode(b)

    b=_pyhl.node(_pyhl.ATTRIBUTE_ID,"/my_compound_attr")
    b.setScalarValue(rinfo_type.size(),rinfo_obj.tostring(),"compound",rinfo_type.hid())
    a.addNode(b)

    rinfo_obj.xsize=99
    rinfo_obj.ysize=109
    b=_pyhl.node(_pyhl.ATTRIBUTE_ID,"/my_compound_attr2")
    b.setScalarValue(rinfo_type.size(),rinfo_obj.tostring(),"compound",rinfo_type.hid())
    a.addNode(b)

    rinfo_obj.xsize=87
    rinfo_obj.ysize=86
    rinfo_obj.xscale=99.9
    area_extent[2]=17.7
    rinfo_obj.area_extent=tuple(area_extent)
    b=_pyhl.node(_pyhl.DATASET_ID,"/my_dataset")
    b.setArrayValue(rinfo_type.size(),[1],rinfo_obj.tostring(),"compound",rinfo_type.hid())
    a.addNode(b)

    a.write("ahetesthdf54_compund_type.hdf")

def verify_compound_object():
    import _rave_info_type
    rinfo_obj=_rave_info_type.object()
    seems_ok = 1
    a=_pyhl.read_nodelist("ahetesthdf54_compund_type.hdf")
    a.selectAll()
    a.fetch()
    b=a.getNode("/my_compound_attr")
    rinfo_obj.fromstring(b.data())
    area_extent = [10.0,20.0,30.0,40.0]

    if not (rinfo_obj.xsize == 10 and rinfo_obj.ysize == 11 and \
	    rinfo_obj.xscale == 140.0 and rinfo_obj.yscale == 160.0 and \
	    rinfo_obj.area_extent==tuple(area_extent)):
	seems_ok=0

    b=a.getNode("/my_compound_attr2")
    rinfo_obj.fromstring(b.data())
    if not (rinfo_obj.xsize == 99 and rinfo_obj.ysize == 109 and \
	    rinfo_obj.xscale == 140.0 and rinfo_obj.yscale == 160.0 and \
	    rinfo_obj.area_extent ==tuple(area_extent)):
	seems_ok=0    

    b=a.getNode("/my_dataset")
    rinfo_obj.fromstring(b.data())
    area_extent[2]=17.7
    if not (rinfo_obj.xsize == 87 and rinfo_obj.ysize == 86 and \
	    rinfo_obj.xscale == 99.9 and rinfo_obj.yscale == 160.0 and \
	    rinfo_obj.area_extent==tuple(area_extent)):
	seems_ok=0

    return seems_ok

def test_function(msg, func):
    if func()==1:
	print "%s SEEMS TO WORK" % (msg)
    else:
	print "%s DOES NOT SEEM TO WORK" % (msg)

def test_it():
    # Write the testdata
    write_int_list()
    write_array_list()
    write_string_list()
    write_compound_object()

    # Test so that what is read is the same as written
    test_function("INTEGER LISTS", verify_int_list)
    test_function("STRING LISTS", verify_string_list)
    test_function("ARRAYS", verify_array_list)
    test_function("COMPOUND OBJECT",verify_compound_object);

    # Clean up after me
    os.unlink("ahetesthdf54_int_list.hdf")
    os.unlink("ahetesthdf54_string_list.hdf")
    os.unlink("ahetesthdf54_arrays.hdf")
    os.unlink("ahetesthdf54_compund_type.hdf")
    
if __name__=="__main__":
    test_it()
